/*
 *                 IFS Research & Development
 *
 *  This program is protected by copyright law and by international
 *  conventions. All licensing, renting, lending or copying (including
 *  for private use), and all other use of the program, which is not
 *  expressively permitted by IFS Research & Development (IFS), is a
 *  violation of the rights of IFS. Such violations will be reported to the
 *  appropriate authorities.
 *
 *  VIOLATIONS OF ANY COPYRIGHT IS PUNISHABLE BY LAW AND CAN LEAD
 *  TO UP TO TWO YEARS OF IMPRISONMENT AND LIABILITY TO PAY DAMAGES.
 * ----------------------------------------------------------------------------
 * File        : IfsWebServices.java
 * Description :
 * Notes       :
 * ----------------------------------------------------------------------------
 * Modified    :
 *    2008-Sep-09  Jacek P - Created
 * ----------------------------------------------------------------------------
 *
 */

package com.ifsworld.servlet;


import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Web Services tester class.
 * Input documents for supported operations:
 *
 *  SimpleWebService.Hello():
 * ==========================
 * <?xml version="1.0" encoding="UTF-8"?>
 * <env:Envelope xmlns:env="http://schemas.xmlsoap.org/soap/envelope/">
 *  <env:Header>
 *  </env:Header>
 *  <env:Body>
 *   <ns1:hello xmlns:ns1="http://webservices.ifsworld.com/">
 *    <name>ABC</name>
 *   </ns1:hello>
 *  </env:Body>
 * </env:Envelope>
 *
 *
 *  Ws2Stateless.SayHello():
 * ==========================
 *
 * <?xml version="1.0" encoding="UTF-8"?>
 * <env:Envelope xmlns:env="http://schemas.xmlsoap.org/soap/envelope/">
 *  <env:Header>
 *  </env:Header>
 *  <env:Body>
 *   <ns1:sayHello xmlns:ns1="http://webservices.ifsworld.com/">
 *    <yourName>ABC</yourName>
 *   </ns1:sayHello>
 *  </env:Body>
 * </env:Envelope>
 *
 */
public class ServiceTester extends HttpServlet
{
   /**
    * Implementation of doPost()
    */
   public void doPost( HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException
   {
      debugRequestHeader(request, "POST");

      String handler = request.getParameter("handler");
      if( !isEmpty(handler) )
      {
         String urlWS = "http://"+request.getLocalName()+":"+request.getLocalPort()+request.getContextPath()+"/"+handler;
         String indata = request.getParameter("source");

         response.setContentType("text/xml; charset=UTF-8");
         OutputStream os = response.getOutputStream();

         URL url = new URL(urlWS);
         URLConnection conn = url.openConnection();
         HttpURLConnection http = (HttpURLConnection)conn;

         http.setDoOutput(true);
         http.setDoInput(true);
         http.setRequestMethod("POST");
         http.setRequestProperty("SOAPAction", "\"\"");
         http.setRequestProperty("Content-Type", "text/xml; charset=UTF-8");

         byte[] dataBytes = indata.getBytes("UTF-8");
         http.setRequestProperty("Content-Length", ""+dataBytes.length);

         debug("Connecting to '"+urlWS+"'");

         http.connect();

         OutputStream requestStream = http.getOutputStream();
         requestStream.write(dataBytes, 0, dataBytes.length);
         requestStream.close();

         int respCode = http.getResponseCode();
         debug("Response code: '"+respCode+"'");

         boolean respOk = respCode==http.HTTP_OK || respCode==http.HTTP_ACCEPTED;

         InputStream is = respOk ? http.getInputStream() : http.getErrorStream();

         BufferedReader in  = new BufferedReader(new InputStreamReader(is));
         BufferedWriter out = new BufferedWriter(new OutputStreamWriter(os));

         String line = null;
         while(true)
         {
            line = in.readLine();
            if( line==null )
               break;
            debug("'"+line+"'");
            out.write(line);
            out.newLine();
         }

         in.close();
         out.flush();
         os.close();
      }
   }

   /**
    * Implementation of doGet().
    */
   public void doGet( HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException
   {
      debugRequestHeader(request, "GET");

      PrintWriter out = response.getWriter();
      String baseUrl   = "http://"+request.getLocalName()+":"+request.getLocalPort()+request.getContextPath()+"/tester";

      try
      {
         String handler = request.getParameter("handler");
         if( isEmpty(handler) )
         {
            out.println("<html><head><title>IFS Web Services tester</title></head><body>");
            out.println("<h2>Test Web Service:</h2>");
            out.println("<a href=\""+baseUrl+"?handler=SimpleWebService\">SimpleWebService.Hello()</a><br>");
            out.println("<a href=\""+baseUrl+"?handler=Ws2Stateless\">Ws2Stateless.SayHello()</a><br>");
            out.println("</body></html>");
         }
         else
         {
            out.println("<html><head><title>IFS Web Services tester</title></head><body>");

            out.println("<h2>Handler: "+handler+"</h2>");
            out.println("<form method=\"POST\" target=\"result\">");
            out.println("<table>");
            out.println("<tr><td><a href=\"javascript:document.forms[0].submit()\">Invoke</a></td><td>&nbsp;</td></tr>");
            out.println("<tr><td><textarea name=\"source\" rows=\"30\" cols=\"60\" wrap=\"off\"><?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            out.println("<env:Envelope xmlns:env=\"http://schemas.xmlsoap.org/soap/envelope/\">");
            out.println(" <env:Header>");
            out.println(" </env:Header>");
            out.println(" <env:Body>");

            if( "SimpleWebService".equals(handler) )
            {
               out.println("  <ns1:hello xmlns:ns1=\"http://webservices.ifsworld.com/\">");
               out.println("   <name>ABC</name>");
               out.println("  </ns1:hello>");
            }
            else if( "Ws2Stateless".equals(handler) )
            {
               out.println("  <ns1:sayHello xmlns:ns1=\"http://webservices.ifsworld.com/\">");
               out.println("   <yourName>ABC</yourName>");
               out.println("  </ns1:sayHello>");
            }
            else
               throw new ServletException("Unsupported Web Service '"+handler+"'");

            out.println(" </env:Body>");
            out.println("</env:Envelope>");
            out.println("</textarea></td><td><iframe name=\"result\" height=\"500\" width=\"500\"></iframe></td></tr>");
            out.println("</table></form></body></html>");
         }
      }
      catch( Exception ex )
      {
         ex.printStackTrace();
         debug("webclient servlet test failed");
         throw new ServletException(ex);
      }
   }

   private static boolean isEmpty( String s )
   {
      return s==null || s.length()==0;
   }

   //==========================================================================
   // Debugging
   //==========================================================================

   /**
    * Prints out all the HTTP header and query string parameters.
    */
   private void debugRequestHeader( HttpServletRequest request, String type )
   {
      debug(type+" request received.");
      StringBuilder buf = new StringBuilder();

      // Read HTTP request headers
      Enumeration en = request.getHeaderNames();
      if( en!=null )
      {
         buf.append("\n   HTTP headers:");
         buf.append("\n  ===============");
         while( en.hasMoreElements() )
         {
            String name = (String)en.nextElement();
            buf.append("\n\t").append(name).append(": ").append(request.getHeader(name));
         }
         buf.append("\n");
      }

      // Read HTTP parameters (from query string or posted data)

      buf.append("\n   Parameters:");
      buf.append("\n  ===============");

      //immutable map with: key -> String, value -> String[]
      Iterator itr = request.getParameterMap().entrySet().iterator();
      while( itr.hasNext() )
      {
         Map.Entry entry = (Map.Entry)itr.next();
         String   key    = (String)   entry.getKey();
         String[] values = (String[]) entry.getValue();

         buf.append("\n\t").append(key).append("=");
         if( values.length==1 )
            buf.append("\"").append(values[0]).append("\"");
         else if( values.length>1 )
         {
            buf.append('{');
            for( int i=0; i<values.length; i++ )
            {
               if( i>0 )
                  buf.append(',');
               buf.append("\"").append(values[i]).append("\"");
            }
            buf.append('}');
         }
      }
      buf.append("\n.");

      if( buf.length() > 0 )
         debug(buf.toString());
   }

   private static void debug( String text )
   {
      System.out.println(text);
   }
}